@@ -1,9 +1,12 @@
 /***************************************************************************
-                              exerciseconvert.h
+                              exercisefactorize.cpp
                              -------------------
     begin                : 2004/06/04
     copyright            : (C) 2004 by Sebastian Stein
     email                : seb.kde@hpfsc.de
+
+    copyright            : (C) 2008 by Tadeu Araujo, tadeu.araujo@ltia.fc.unesp.br
+    copyright            : (C) 2008 by Danilo Balzaque, danilo.balzaque@ltia.fc.unesp.br
  ***************************************************************************/
 
 /***************************************************************************
@@ -19,23 +22,29 @@
 #include "exercisefactorize.moc"
 
 /* these includes are needed for KDE support */
-#include <kapplication.h>
+#include <klineedit.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <knumvalidator.h>
 
 /* these includes are needed for Qt support */
+#include <qapplication.h>
 #include <qlabel.h>
 #include <qlayout.h>
-#include <qlineedit.h>
 #include <qpushbutton.h>
-#include <qtooltip.h>
-#include <qwhatsthis.h>
+ #include <QToolButton>
+ #include <qpainter.h>
+ 
+//Added by qt3to4:
+#include <QGridLayout>
+#include <QHBoxLayout>
+#include <QVBoxLayout>
+#include <QKeyEvent>
+#include <QPaintEvent>
 
 /* standard C++ library includes */
 #include <stdlib.h>
 
-#include "factorizedwidget.h"
 #include "primenumber.h"
 #include "rationalwidget.h"
 #include "resultwidget.h"
@@ -44,17 +53,15 @@
 /* ----- public member functions ----- */
 
 /* constructor */
-ExerciseFactorize::ExerciseFactorize(QWidget * parent, const char * name):
-		ExerciseBase(parent, name)
+ExerciseFactorize::ExerciseFactorize(QWidget * parent):
+		ExerciseBase(parent)
 {
-	QPalette pal;
-	QColorGroup cg;
 #ifdef DEBUG
-	kdDebug() << "constructor ExerciseFactorize()" << endl;
+	kDebug() << "constructor ExerciseFactorize()";
 #endif
 
 	/* create a new task */
-	QApplication::setOverrideCursor(waitCursor); /* show the sand clock */
+	QApplication::setOverrideCursor(Qt::WaitCursor); /* show the sand clock */
 	createTask();
 	QApplication::restoreOverrideCursor(); /* show the normal cursor */
 
@@ -62,82 +69,135 @@ ExerciseFactorize::ExerciseFactorize(QWidget * parent, const char * name):
 	// result
 	m_currentState = _CHECK_TASK;
 
-	Form1Layout = new QVBoxLayout( this, 11, 6, "Form1Layout"); 
+	QFont defaultFont = SettingsClass::taskFont();
 
-	layout9 = new QVBoxLayout( 0, 0, 6, "layout9"); 
+	taskWidget = new QWidget(this);
+	taskWidget->setObjectName("taskWidget");
+	checkWidget = new QWidget(this);
+	checkWidget->setObjectName("checkWidget");
 	
-	layout4 = new QHBoxLayout( 0, 0, 6, "layout4"); 
-	
-	m_taskLabel = new QLabel( this, "m_taskLabel" );
-	layout4->addWidget( m_taskLabel );
+	baseGrid = new QGridLayout(this);
+	baseGrid->setObjectName( "baseGrid" );
+	baseGrid->setColumnStretch(0,1);
 
-	m_equalSignLabel = new QLabel( this, "m_equalSignLabel" );
-	layout4->addWidget( m_equalSignLabel );
+	baseGrid->addWidget(taskWidget, 0, 0);
+	baseGrid->addWidget(checkWidget, 0, 1);
 	
-	m_factorsEnteredEdit = new QLineEdit( this, "m_factorsEnteredEdit" );
-	layout4->addWidget( m_factorsEnteredEdit );
+	taskLayout = new QGridLayout(this);
+	taskLayout->setObjectName( "taskLayout" );
+	taskLayout->setRowStretch(0,1);
+	taskLayout->setRowStretch(6,1);
+	taskLayout->setColumnStretch(0,1);
+	taskLayout->setColumnStretch(7,1);
+
+	checkLayout = new QGridLayout(this);
+	checkLayout->setObjectName( "checkLayout" );
+
+	m_taskLabel = new QLabel( this );
+	m_taskLabel->setObjectName( "taskLabel" );
+	m_taskLabel->setFont(defaultFont);	
+	taskLayout->addWidget( m_taskLabel, 1, 1 );
+
+	defaultFont.setBold( true );
+	defaultFont.setPointSize(16);
+
+	m_equalSignLabel = new QLabel( this );
+	m_equalSignLabel->setObjectName( "equalSignLabel" );
+	m_equalSignLabel->setText("=");
+	m_equalSignLabel->setFont(defaultFont);
+	taskLayout->addWidget( m_equalSignLabel, 1, 2 );
+
+	m_factorsEnteredEdit = new KLineEdit( this );
+	m_factorsEnteredEdit->setObjectName( "factorsEnteredEdit" );
 	m_factorsEnteredEdit->setReadOnly(true);
-	m_factorsEnteredEdit->setEnabled(false);
-	m_factorsEnteredEdit->setPaletteForegroundColor(QColor(0, 0, 0));
-	
-	m_factorsWidget = new FactorizedWidget( this, "m_factorsWidget", m_factorsResult);
-	layout4->addWidget( m_factorsWidget );
-	m_factorsWidget->hide();
-	
-	result_label = new QLabel( this, "result_label" );
-	layout4->addWidget( result_label );
-	spacer1 = new QSpacerItem( 25, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
-	layout4->addItem( spacer1 );
+	QPalette pal;
+	pal.setColor( m_factorsEnteredEdit->foregroundRole(), QColor(0, 0, 0) );
+	m_factorsEnteredEdit->setPalette(pal);
+	m_factorsEnteredEdit->setFont(defaultFont);
+	m_factorsEnteredEdit->setFixedSize(320,35);	
+	taskLayout->addWidget( m_factorsEnteredEdit, 1, 3, 1, 5 );
 
-	layout9->addLayout( layout4 );
-	spacer2 = new QSpacerItem( 20, 21, QSizePolicy::Minimum, QSizePolicy::Expanding );
-	layout9->addItem( spacer2 );
-	
-	layout2 = new QVBoxLayout( 0, 0, 6, "layout2"); 
-	
-	layout1 = new QGridLayout( 0, 1, 1, 0, 6, "layout1"); 
-	
-	m_factor3Button = new QPushButton( this, "m_factor3Button" );
-	layout1->addWidget( m_factor3Button, 0, 1 );
-	
-	m_factor2Button = new QPushButton( this, "m_factor2Button" );
-	layout1->addWidget( m_factor2Button, 0, 0 );
-	
-	m_factor13Button = new QPushButton( this, "m_factor13Button" );
-	layout1->addWidget( m_factor13Button, 1, 1 );
-	
-	m_factor7Button = new QPushButton( this, "m_factor7Button" );
-	layout1->addWidget( m_factor7Button, 0, 3 );
-	
-	m_factor11Button = new QPushButton( this, "m_factor11Button" );
-	layout1->addWidget( m_factor11Button, 1, 0 );
-	
-	m_factor19Button = new QPushButton( this, "m_factor19Button" );
-	layout1->addWidget( m_factor19Button, 1, 3 );
-	
-	m_factor5Button = new QPushButton( this, "m_factor5Button" );
-	layout1->addWidget( m_factor5Button, 0, 2 );
-	
-	m_factor17Button = new QPushButton( this, "m_factor17Button" );
-	layout1->addWidget( m_factor17Button, 1, 2 );
+	defaultFont.setPointSize(10);
 
-	layout2->addLayout( layout1 );
+	m_factor2Button = new QPushButton( this );
+	m_factor2Button->setObjectName( "factor2Button" );
+	m_factor2Button->setFixedSize(50,35);	
+	m_factor2Button->setFont(defaultFont);
+	taskLayout->addWidget( m_factor2Button, 4, 3 );
 	
-	m_removeLastFactorButton = new QPushButton( this, "m_removeLastFactorButton" );
-	layout2->addWidget( m_removeLastFactorButton );
-	layout9->addLayout( layout2 );
+	m_factor3Button = new QPushButton( this );
+	m_factor3Button->setObjectName( "factor3Button" );
+	m_factor3Button->setFixedSize(50,35);		
+	m_factor3Button->setFont(defaultFont);
+	taskLayout->addWidget( m_factor3Button, 4, 4 );
 
-	spacer4 = new QSpacerItem( 20, 21, QSizePolicy::Minimum, QSizePolicy::Expanding );
-	layout9->addItem( spacer4 );
+	m_factor5Button = new QPushButton( this );
+	m_factor5Button->setObjectName( "factor5Button" );
+	m_factor5Button->setFixedSize(50,35);
+	m_factor5Button->setFont(defaultFont);
+	taskLayout->addWidget( m_factor5Button, 4, 5 );
+
+	m_factor7Button = new QPushButton( this );
+	m_factor7Button->setObjectName( "factor7Button" );
+	m_factor7Button->setFixedSize(50,35);
+	m_factor7Button->setFont(defaultFont);
+	taskLayout->addWidget( m_factor7Button, 4, 6 );
+
+	m_factor11Button = new QPushButton( this );
+	m_factor11Button->setObjectName( "factor11Button" );
+	m_factor11Button->setFixedSize(50,35);
+	m_factor11Button->setFont(defaultFont);
+	taskLayout->addWidget( m_factor11Button, 5, 3 );
 	
-	layout7 = new QHBoxLayout( 0, 0, 6, "layout7"); 
-	spacer3 = new QSpacerItem( 361, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
-	layout7->addItem( spacer3 );
-	
-	m_checkButton = new QPushButton( this, "m_checkButton" );
-	layout7->addWidget( m_checkButton );
-	layout9->addLayout( layout7 );
-	Form1Layout->addLayout( layout9 );
+	m_factor13Button = new QPushButton( this );
+	m_factor13Button->setObjectName( "factor13Button" );
+	m_factor13Button->setFixedSize(50,35);
+	m_factor13Button->setFont(defaultFont);
+	taskLayout->addWidget( m_factor13Button, 5, 4 );
+
+	m_factor17Button = new QPushButton( this );
+	m_factor17Button->setObjectName( "factor17Button" );
+	m_factor17Button->setFixedSize(50,35);
+	m_factor17Button->setFont(defaultFont);
+	taskLayout->addWidget( m_factor17Button, 5, 5 );
+
+	m_factor19Button = new QPushButton( this );
+	m_factor19Button->setObjectName( "factor19Button" );
+	m_factor19Button->setFixedSize(50,35);
+	m_factor19Button->setFont(defaultFont);
+	taskLayout->addWidget( m_factor19Button, 5, 6 );
+
+	m_removeLastFactorButton = new QPushButton( this );
+	m_removeLastFactorButton->setObjectName( "removeLastFactor" );
+	m_removeLastFactorButton->setFixedSize(115,35);
+	m_removeLastFactorButton->setFont(defaultFont);
+	m_removeLastFactorButton->setText(i18n("Backspace"));
+	taskLayout->addWidget( m_removeLastFactorButton, 3, 4, 1, 3, Qt::AlignRight );
+
+	defaultFont.setPointSize(10);
+
+	m_skipButton = new QPushButton( this );
+	m_skipButton->setObjectName( "m_skipButton" );
+	m_skipButton->setText(i18n("&Skip"));
+	m_skipButton->setToolTip(i18n("Click on this button to skip this question."));
+	m_skipButton->setFont(defaultFont);		
+	QObject::connect(m_skipButton, SIGNAL(clicked()), this, SLOT(slotSkipButtonClicked()));
+	checkLayout->addWidget(m_skipButton, 1, 1);	
+
+	// the check task button
+	m_checkButton = new QPushButton( this );
+	m_checkButton->setObjectName( "m_checkButton" );
+	m_checkButton->setText( i18n( "&Check" ) );
+	m_checkButton->setToolTip(i18n("Click on this button to check your result. The button will not work if you have not entered a result yet."));
+	QObject::connect(m_checkButton, SIGNAL(clicked()), this, SLOT(slotCheckButtonClicked()));
+	m_checkButton->setFont(defaultFont);		
+	checkLayout->addWidget(m_checkButton, 1, 0);			
+	m_checkButton->setDefault(true); // is the default button of the dialog
+
+	// next is the result widget
+	m_resultWidget = new ResultWidget( this,  m_factorsResult );
+	m_resultWidget->setObjectName("m_resultWidget");
+	checkLayout->addWidget(m_resultWidget, 0, 0, 1, 2);
 
 	// the current task
 	QString tmp_str;
@@ -145,17 +205,14 @@ ExerciseFactorize::ExerciseFactorize(QWidget * parent, const char * name):
 	m_taskLabel->setText(tmp_str);
 
 	// now set the color for the task label
-	m_taskLabel->setPaletteForegroundColor(SettingsClass::numberColor());
-
-	// the equal sign
-	m_equalSignLabel->setText("=");
+	pal = QPalette();
+	pal.setColor( m_taskLabel->foregroundRole(), SettingsClass::numberColor() );
+	m_taskLabel->setPalette( pal );
 
 	// now set the color for the equal sign
-	m_equalSignLabel->setPaletteForegroundColor(SettingsClass::operationColor());
-
-	// the wrong/correct label, we hide it
-	result_label->setText(i18n("WRONG"));
-	result_label->hide();
+	pal = QPalette();
+	pal.setColor( m_equalSignLabel->foregroundRole(), SettingsClass::operationColor() );
+	m_equalSignLabel->setPalette( pal );
 
 	// the prime factor buttons
 	m_factor2Button->setText( i18n( "2" ) );
@@ -176,49 +233,50 @@ ExerciseFactorize::ExerciseFactorize(QWidget * parent, const char * name):
 	QObject::connect(m_factor19Button, SIGNAL(clicked()), this, SLOT(slotFactor19ButtonClicked()));
 
 	// add tooltips to the factor buttons
-	QToolTip::add(m_factor2Button, i18n("Add prime factor 2."));
-	QToolTip::add(m_factor3Button, i18n("Add prime factor 3."));
-	QToolTip::add(m_factor5Button, i18n("Add prime factor 5."));
-	QToolTip::add(m_factor7Button, i18n("Add prime factor 7."));
-	QToolTip::add(m_factor11Button, i18n("Add prime factor 11."));
-	QToolTip::add(m_factor13Button, i18n("Add prime factor 13."));
-	QToolTip::add(m_factor17Button, i18n("Add prime factor 17."));
-	QToolTip::add(m_factor19Button, i18n("Add prime factor 19."));
+	m_factor2Button->setToolTip(i18n("Add prime factor 2."));
+	m_factor3Button->setToolTip(i18n("Add prime factor 3."));
+	m_factor5Button->setToolTip(i18n("Add prime factor 5."));
+	m_factor7Button->setToolTip(i18n("Add prime factor 7."));
+	m_factor11Button->setToolTip(i18n("Add prime factor 11."));
+	m_factor13Button->setToolTip(i18n("Add prime factor 13."));
+	m_factor17Button->setToolTip(i18n("Add prime factor 17."));
+	m_factor19Button->setToolTip(i18n("Add prime factor 19."));
 
 	// the remove last factor button
-	m_removeLastFactorButton->setText( i18n( "&Remove Last Factor" ) );
 	m_removeLastFactorButton->setEnabled(false);
 	QObject::connect(m_removeLastFactorButton, SIGNAL(clicked()), this, SLOT(slotRemoveLastFactorButtonClicked()));
-	QToolTip::add(m_removeLastFactorButton, i18n("Removes the last entered prime factor."));
+	m_removeLastFactorButton->setToolTip(i18n("Removes the last entered prime factor."));
+	
+	m_factor2Button->setFocusPolicy( Qt::NoFocus );
+	m_factor3Button->setFocusPolicy( Qt::NoFocus );
+	m_factor5Button->setFocusPolicy( Qt::NoFocus );
+	m_factor7Button->setFocusPolicy( Qt::NoFocus );
+	m_factor11Button->setFocusPolicy( Qt::NoFocus );
+	m_factor13Button->setFocusPolicy( Qt::NoFocus );
+	m_factor17Button->setFocusPolicy( Qt::NoFocus );
+	m_factor19Button->setFocusPolicy( Qt::NoFocus );	
+	m_removeLastFactorButton->setFocusPolicy( Qt::NoFocus );	
+	m_skipButton->setFocusPolicy( Qt::NoFocus );	
+	m_checkButton->setFocusPolicy( Qt::NoFocus );	
 
-	// the check task button
-	m_checkButton->setText( i18n( "&Check Task" ) );
-	QToolTip::add(m_checkButton, i18n("Click on this button to check your result. The button will not work if you have not entered a result yet."));
-	QObject::connect(m_checkButton, SIGNAL(clicked()), this, SLOT(slotCheckButtonClicked()));
-	m_checkButton->setDefault(true); // is the default button of the dialog
+	m_checkButton->setDefault( true );
 
-	// that the user can start choosing the prime factors
-	m_factor2Button->setFocus();
-
-	// set the tab order
-	setTabOrder(m_factor2Button, m_factor3Button);
-	setTabOrder(m_factor3Button, m_factor5Button);
-	setTabOrder(m_factor5Button, m_factor7Button);
-	setTabOrder(m_factor7Button, m_factor11Button);
-	setTabOrder(m_factor13Button, m_factor17Button);
-	setTabOrder(m_factor17Button, m_factor19Button);
-	setTabOrder(m_factor19Button, m_removeLastFactorButton);
+	setLayout(baseGrid);
+	taskWidget->setLayout(taskLayout);
+  	checkWidget->setLayout(checkLayout);
 
 	// add tooltip and qwhatsthis help to the widget
-	QToolTip::add(this, i18n("In this exercise you have to factorize a given number."));
-	QWhatsThis::add(this, i18n("In this exercise you have to factorize a given number. You have to enter all prime factors of the number. You can add a prime factor by clicking on the corresponding button. The chosen prime factors will be shown in the input field. Do not forget to enter all prime factors, even when a prime factor repeats several times!"));
+	setToolTip(i18n("In this exercise you have to factorize a given number."));
+	setWhatsThis( i18n("In this exercise you have to factorize a given number. You have to enter all prime factors of the number. You can add a prime factor by clicking on the corresponding button. The chosen prime factors will be shown in the input field. Do not forget to enter all prime factors, even when a prime factor repeats several times."));
+
+	m_edit = true;
 }
 
 /* destructor */
 ExerciseFactorize::~ExerciseFactorize()
 {
 #ifdef DEBUG
-	kdDebug() << "destructor ExerciseFactorize()" << endl;
+	kDebug() << "destructor ExerciseFactorize()";
 #endif
 
 	/* no need to delete any child widgets, Qt does it by itself */
@@ -230,16 +288,16 @@ ExerciseFactorize::~ExerciseFactorize()
 void ExerciseFactorize::forceNewTask()
 {
 #ifdef DEBUG
-	kdDebug() << "forceNewTask ExerciseFactorize()" << endl;
+	kDebug() << "forceNewTask ExerciseFactorize()";
 #endif
 
 	if (m_currentState == _CHECK_TASK)
 	{
-		// emit the signal for wrong
-		signalExerciseSolvedWrong();
+		// emit the signal for skipped
+		signalExerciseSkipped();
 	}
 	m_currentState = _CHECK_TASK;
-	m_checkButton->setText(i18n("&Check Task"));
+	m_checkButton->setText(i18n("&Check"));
 
 	// generate next task
 	(void) nextTask();
@@ -251,13 +309,17 @@ void ExerciseFactorize::forceNewTask()
 void ExerciseFactorize::update()
 {
 	// now set the color for the task label
-	m_taskLabel->setPaletteForegroundColor(SettingsClass::numberColor());
+	QPalette pal;
+	pal.setColor( m_taskLabel->foregroundRole(), SettingsClass::numberColor() );
+	m_taskLabel->setPalette( pal );
 
 	// now set the color for the equal sign
-	m_equalSignLabel->setPaletteForegroundColor(SettingsClass::operationColor());
+	pal = QPalette();
+	pal.setColor( m_equalSignLabel->foregroundRole(), SettingsClass::operationColor() );
+	m_equalSignLabel->setPalette( pal );
 
 	// and the factors
-	m_factorsWidget->updateAndRepaint();
+	m_resultWidget->update();
 
 	// update for itself
 	((QWidget *) this)->update();
@@ -273,7 +335,7 @@ void ExerciseFactorize::createTask()
 
 	// just pick one number out of the possible numbers to factorize
 	m_taskNumber = possibleTasks[uint((double(rand()) / RAND_MAX) * numberPossibleTasks)];
-	
+
 	// now get the primefactors of the taskNumber
 	m_factorsResult.clear();
 	uint_number = m_taskNumber;
@@ -301,11 +363,10 @@ void ExerciseFactorize::showResult()
 {
 	QString tmp_str, tmp_str2; /* to build a string for a label */
 	QPalette pal;
-	QColorGroup cg;
 	uint uint_result = 0;
 
 	// change the tooltip of the check button
-	QToolTip::add(m_checkButton, i18n("Click on this button to get to the next task."));
+	m_checkButton->setToolTip(i18n("Click on this button to get to the next question."));
 
 	// disable prime factor buttons
 	m_factor2Button->setEnabled(false);
@@ -316,16 +377,16 @@ void ExerciseFactorize::showResult()
 	m_factor13Button->setEnabled(false);
 	m_factor17Button->setEnabled(false);
 	m_factor19Button->setEnabled(false);
+	m_skipButton->setEnabled(false);		
 
 	// disable factor removal button as well
 	m_removeLastFactorButton->setEnabled(false);
 
 	// show the result
-	m_factorsWidget->setFactors(m_factorsResult);
-	m_factorsWidget->show();
+	m_resultWidget->setFactors(m_factorsResult);
 
 	// now calculate the product of the prime factors entered by the user
-	for (uint tmp_uint = 0; tmp_uint < m_factorsEntered.count(); tmp_uint++)
+	for (int tmp_uint = 0; tmp_uint < m_factorsEntered.count(); tmp_uint++)
 	{
 		if (tmp_uint == 0)
 		{
@@ -341,33 +402,16 @@ void ExerciseFactorize::showResult()
 		signalExerciseSolvedCorrect();
 
 		/* yes, the user entered the correct result */
-		result_label->setText(i18n("CORRECT"));
-		pal = result_label->palette(); /* set green font color */
-		cg = pal.active();
-		cg.setColor(QColorGroup::Foreground, QColor(6, 179, 0));
-		pal.setActive(cg);
-		cg = pal.inactive();
-		cg.setColor(QColorGroup::Foreground, QColor(6, 179, 0));
-		pal.setInactive(cg);
-		result_label->setPalette(pal);
+		m_resultWidget->setResult( ratio(), 1 );
 	} else {
 		// emit the signal for wrong
 		signalExerciseSolvedWrong();
 
 		/* no, the user entered the wrong result */
-		result_label->setText(i18n("WRONG"));
-		pal = result_label->palette(); /* set red font color */
-		cg = pal.active();
-		cg.setColor(QColorGroup::Foreground, QColor(red));
-		pal.setActive(cg);
-		cg = pal.inactive();
-		cg.setColor(QColorGroup::Foreground, QColor(red));
-		pal.setInactive(cg);
-		result_label->setPalette(pal);
-
+		m_resultWidget->setResult( ratio(), 0 );
 	} /* if (entered_result == result) */
 
-	result_label->show(); /* show the result at the end of the task */
+	m_edit = false;
 
 	return;
 }
@@ -376,7 +420,7 @@ void ExerciseFactorize::showResult()
 void ExerciseFactorize::nextTask()
 {
 	// change the tooltip of the check button
-	QToolTip::add(m_checkButton, i18n("Click on this button to check your result. The button will not work if you have not entered a result yet."));
+	m_checkButton->setToolTip(i18n("Click on this button to check your result. The button will not work if you have not entered a result yet."));
 
 	// enable prime factor buttons
 	m_factor2Button->setEnabled(true);
@@ -387,22 +431,22 @@ void ExerciseFactorize::nextTask()
 	m_factor13Button->setEnabled(true);
 	m_factor17Button->setEnabled(true);
 	m_factor19Button->setEnabled(true);
+	m_skipButton->setEnabled(true);	
 
 	// disable the factor removal button, there are no factors to be removed yet
 	m_removeLastFactorButton->setEnabled(false);
 
-	result_label->hide(); /* do not show the result at the end of the task */
-	m_factorsWidget->hide();
+	m_resultWidget->setResult( ratio(), -1 );
 
 	/* clear user input */
 	m_factorsEntered.clear();
 	m_factorsResult.clear();
 	m_factorsEnteredEdit->setText("");
 
-	m_factor2Button->setFocus();
+	m_factorsEnteredEdit->setFocus();
 
 	/* create a new task */
-	QApplication::setOverrideCursor(waitCursor); /* show the sand clock */
+	QApplication::setOverrideCursor(Qt::WaitCursor); /* show the sand clock */
 	createTask();
 	QApplication::restoreOverrideCursor(); /* show the normal cursor */
 
@@ -411,6 +455,8 @@ void ExerciseFactorize::nextTask()
 	tmp_str.setNum(m_taskNumber);
 	m_taskLabel->setText(tmp_str);
 
+	m_edit = true;
+
 	return;
 }
 
@@ -437,8 +483,8 @@ void ExerciseFactorize::updateEnteredEdit()
 	// find the end of the list
 	uintList::iterator it;
 
-	
-	for (it = m_factorsEntered.begin(); it != m_factorsEntered.end(); it++)
+
+	for (it = m_factorsEntered.begin(); it != m_factorsEntered.end(); ++it)
 	{
 		// convert the factor into a string
 		str_tmp.setNum(*it);
@@ -454,7 +500,7 @@ void ExerciseFactorize::updateEnteredEdit()
 
 	// set the text into the line edit
 	m_factorsEnteredEdit->setText(str_output);
-	
+
 	return;
 }
 
@@ -468,14 +514,22 @@ void ExerciseFactorize::slotCheckButtonClicked()
 		if (m_factorsEntered.count() == 0)
 			return;
 		m_currentState = _NEXT_TASK;
-		m_checkButton->setText(i18n("N&ext Task"));
+		m_checkButton->setText(i18n("N&ext"));
 		(void) showResult();
 	} else {
 		m_currentState = _CHECK_TASK;
-		m_checkButton->setText(i18n("&Check Task"));
+		m_checkButton->setText(i18n("&Check"));
 		(void) nextTask();
 	}
 
+	// update the line edit
+	updateEnteredEdit();
+	return;
+}
+
+void ExerciseFactorize::slotSkipButtonClicked()
+{
+	forceNewTask();
 	// update the line edit
 	updateEnteredEdit();
 
@@ -547,7 +601,7 @@ void ExerciseFactorize::slotRemoveLastFactorButtonClicked()
 		--it;
 
 		// remove last item
-		m_factorsEntered.remove(it);
+		m_factorsEntered.erase(it);
 	}
 
 	// if the list is empty, we have to disable the delete button
@@ -559,3 +613,92 @@ void ExerciseFactorize::slotRemoveLastFactorButtonClicked()
 
 	return;
 }
+
+/* ------ protected events ------ */
+void ExerciseFactorize::keyPressEvent(QKeyEvent * e)
+{
+	if (m_edit == false)
+		return;
+
+	switch (e->key())
+	{
+		case Qt::Key_1:
+			if (m_buffer != 1)
+			{
+				m_buffer = 1;
+			}
+			else
+			{
+				m_buffer = 0;
+				addFactor(11);
+			}
+			break;
+
+		case Qt::Key_2:
+			m_buffer = 0;
+			addFactor(2);
+			break;
+
+		case Qt::Key_3:
+			if (m_buffer != 1)
+			{
+				addFactor(3);
+			}
+			else
+			{
+				addFactor(13);
+			}
+			m_buffer = 0;
+			break;
+
+		case Qt::Key_5:
+			m_buffer = 0;
+			addFactor(5);
+			break;
+
+		case Qt::Key_7:
+			if (m_buffer == 1)
+			{
+				addFactor(17);
+			}
+			else
+			{
+				addFactor(7);
+			}
+			m_buffer = 0;
+			break;
+
+		case Qt::Key_9:
+			if (m_buffer == 1)
+			{
+				addFactor(19);
+			}
+			m_buffer = 0;
+			break;
+
+		case Qt::Key_Delete:
+		case Qt::Key_Backspace:
+			m_buffer = 0;
+			slotRemoveLastFactorButtonClicked();
+			break;
+
+		case Qt::Key_Return:
+			m_buffer = 0;
+			slotCheckButtonClicked();
+			break;
+
+		default:
+			m_buffer = 0;
+			break;
+	}
+}
+
+void ExerciseFactorize::showEvent( QShowEvent * event )
+{
+#ifdef DEBUG
+	kDebug() << "ExerciseFactorize::showEvent()";
+#endif
+
+	if ( isVisible() )
+		m_factorsEnteredEdit->setFocus();
+}
