@@ -12,6 +12,8 @@
 #include "KmlFileViewItem.h"
 #include "GeoDataDocument.h"    // In geodata/data/
 
+using namespace Marble;
+
 KmlFileViewItem::KmlFileViewItem( PlaceMarkManager& manager, GeoDataDocument& document ) :
     m_placeMarkManager( manager ),
     m_document( document )
@@ -35,7 +37,10 @@ void KmlFileViewItem::closeFile()
 
 QVariant KmlFileViewItem::data() const
 {
-    return m_document.name();
+    if(!m_document.name().isEmpty())
+        return m_document.name();
+    else
+        return QString("KML Document"); // this should be replaced by the file name later
 }
 
 bool KmlFileViewItem::isShown() const
