@@ -5,7 +5,7 @@
 // find a copy of this license in LICENSE.txt in the top directory of
 // the source code.
 //
-// Copyright 2008      Inge Wallin  <ingwa@kde.org>"
+// Copyright 2008      Inge Wallin  <ingwa@kde.org>
 //
 
 
@@ -23,6 +23,8 @@
 // Qt
 #include <QtGui/QRegion>
 
+namespace Marble
+{
 
 class AbstractProjectionHelperPrivate
 {
@@ -30,7 +32,7 @@ class AbstractProjectionHelperPrivate
     QRegion  activeRegion;
 };
 
-
+}
 
 
 #endif // ABSTRACTPROJECTIONHELPER_P_H
